@@ -15,6 +15,8 @@ use handle::{self, RsvgHandle};
 use state::State;
 use util::utf8_cstr;
 
+pub enum RsvgCssStyles {}
+
 struct Declaration {
     prop_value: String,
     important: bool,
@@ -84,7 +86,11 @@ struct DocHandlerData {
     selector: *mut CRSelector,
 }
 
-fn parse_into_handle(handle: *mut RsvgHandle, buf: &str) {
+pub fn parse_into_handle(handle: *mut RsvgHandle, buf: &str) {
+    if buf.len() == 0 {
+        return; // libcroco doesn't like empty strings :(
+    }
+
     unsafe {
         let handler_data = DocHandlerData {
             handle,
@@ -217,12 +223,6 @@ unsafe extern "C" fn css_property(
 
 unsafe extern "C" fn css_error(_a_this: *mut CRDocHandler) {
     println!("CSS parsing error");
-    
-    let handler_data = get_doc_handler_data(_a_this);
-    if !handler_data.selector.is_null() && ((*(handler_data.selector)).ref_count > 1) {
-        cr_selector_unref(handler_data.selector);
-        handler_data.selector = ptr::null_mut();
-    }
 }
 
 unsafe extern "C" fn css_unrecoverable_error(_a_this: *mut CRDocHandler) {
@@ -230,14 +230,15 @@ unsafe extern "C" fn css_unrecoverable_error(_a_this: *mut CRDocHandler) {
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_css_styles_new() -> *mut CssStyles {
-    Box::into_raw(Box::new(CssStyles::new()))
+pub extern "C" fn rsvg_css_styles_new() -> *mut RsvgCssStyles {
+    Box::into_raw(Box::new(CssStyles::new())) as *mut RsvgCssStyles
 }
 
 #[no_mangle]
-pub unsafe extern "C" fn rsvg_css_styles_free(raw_styles: *mut CssStyles) {
+pub unsafe extern "C" fn rsvg_css_styles_free(raw_styles: *mut RsvgCssStyles) {
     assert!(!raw_styles.is_null());
-    Box::from_raw(raw_styles);
+
+    Box::from_raw(raw_styles as *mut CssStyles);
 }
 
 #[no_mangle]
