@@ -1,4 +1,3 @@
-use libc;
 use std::collections::HashMap;
 
 use attributes::Attribute;
@@ -23,7 +22,6 @@ use filters::{
     turbulence::Turbulence,
 };
 use gradient::NodeGradient;
-use handle::RsvgHandle;
 use image::NodeImage;
 use link::NodeLink;
 use marker::NodeMarker;
@@ -34,8 +32,8 @@ use property_bag::PropertyBag;
 use shapes::{NodeCircle, NodeEllipse, NodeLine, NodePath, NodePoly, NodeRect};
 use stop::NodeStop;
 use structure::{NodeDefs, NodeGroup, NodeSvg, NodeSwitch, NodeSymbol, NodeUse};
+use style::NodeStyle;
 use text::{NodeTRef, NodeTSpan, NodeText};
-use util::utf8_cstr;
 
 macro_rules! node_create_fn {
     ($name:ident, $node_type:ident, $new_fn:expr) => {
@@ -43,9 +41,9 @@ macro_rules! node_create_fn {
             element_name: &str,
             id: Option<&str>,
             class: Option<&str>,
-            parent: *const RsvgNode,
-        ) -> *const RsvgNode {
-            boxed_node_new(
+            parent: Option<&RsvgNode>,
+        ) -> RsvgNode {
+            node_new(
                 NodeType::$node_type,
                 parent,
                 element_name,
@@ -162,6 +160,7 @@ node_create_fn!(
 );
 node_create_fn!(create_spot_light, LightSource, LightSource::new_spot_light);
 node_create_fn!(create_stop, Stop, NodeStop::new);
+node_create_fn!(create_style, Style, NodeStyle::new);
 node_create_fn!(create_svg, Svg, NodeSvg::new);
 node_create_fn!(create_switch, Switch, NodeSwitch::new);
 node_create_fn!(create_symbol, Symbol, NodeSymbol::new);
@@ -176,7 +175,8 @@ node_create_fn!(
 );
 node_create_fn!(create_use, Use, NodeUse::new);
 
-type NodeCreateFn = fn(&str, Option<&str>, Option<&str>, *const RsvgNode) -> *const RsvgNode;
+type NodeCreateFn =
+    fn(name: &str, id: Option<&str>, class: Option<&str>, parent: Option<&RsvgNode>) -> RsvgNode;
 
 lazy_static! {
     // Lines in comments are elements that we don't support.
@@ -253,7 +253,7 @@ lazy_static! {
         /* h.insert("script",           (false, as NodeCreateFn)); */
         /* h.insert("set",              (false, as NodeCreateFn)); */
         h.insert("stop",                (true,  create_stop as NodeCreateFn));
-        /* h.insert("style",            (false, as NodeCreateFn)); */
+        h.insert("style",               (false, create_style as NodeCreateFn));
         h.insert("subImage",            (false, create_group as NodeCreateFn));
         h.insert("subImageRef",         (false, create_image as NodeCreateFn));
         h.insert("svg",                 (true,  create_svg as NodeCreateFn));
@@ -271,21 +271,12 @@ lazy_static! {
     };
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_load_new_node(
-    raw_name: *const libc::c_char,
-    parent: *const RsvgNode,
-    pbag: *const PropertyBag<'_>,
-    defs: *mut Defs,
-) -> *const RsvgNode {
-    assert!(!raw_name.is_null());
-    assert!(!pbag.is_null());
-    assert!(!defs.is_null());
-
-    let name = unsafe { utf8_cstr(raw_name) };
-    let pbag = unsafe { &*pbag };
-    let defs = unsafe { &mut *defs };
-
+pub fn rsvg_load_new_node(
+    name: &str,
+    parent: Option<&RsvgNode>,
+    pbag: &PropertyBag,
+    defs: &mut Defs,
+) -> RsvgNode {
     let mut id = None;
     let mut class = None;
 
@@ -310,53 +301,10 @@ pub extern "C" fn rsvg_load_new_node(
     };
 
     let node = create_fn(name, id, class, parent);
-    assert!(!node.is_null());
 
     if id.is_some() {
-        let n = unsafe { &*node };
-        defs.insert(id.unwrap(), n);
+        defs.insert(id.unwrap(), &node);
     }
 
     node
 }
-
-#[no_mangle]
-pub extern "C" fn rsvg_load_set_node_atts(
-    handle: *const RsvgHandle,
-    raw_node: *mut RsvgNode,
-    pbag: *const PropertyBag<'_>,
-) {
-    assert!(!raw_node.is_null());
-    assert!(!pbag.is_null());
-
-    let node: &RsvgNode = unsafe { &*raw_node };
-    let pbag = unsafe { &*pbag };
-
-    node.set_atts(node, handle, pbag);
-
-    // The "svg" node is special; it will load its id/class
-    // attributes until the end, when sax_end_element_cb() calls
-    // rsvg_node_svg_apply_atts()
-    if node.get_type() != NodeType::Svg {
-        node.set_style(handle, pbag);
-    }
-
-    node.set_overridden_properties();
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_load_set_svg_node_atts(
-    handle: *const RsvgHandle,
-    raw_node: *const RsvgNode,
-) {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    if node.get_type() != NodeType::Svg {
-        return;
-    }
-
-    node.with_impl(|svg: &NodeSvg| {
-        svg.set_delayed_style(node, handle);
-    });
-}
