@@ -1,5 +1,6 @@
 use cairo::{Matrix, MatrixTrait};
 use downcast_rs::*;
+use glib;
 use glib::translate::*;
 use glib_sys;
 use std::cell::{Cell, Ref, RefCell};
@@ -200,6 +201,7 @@ pub enum NodeType {
     RadialGradient,
     Rect,
     Stop,
+    Style,
     Svg,
     Switch,
     Symbol,
@@ -397,9 +399,11 @@ impl Node {
                     }
 
                     Attribute::SystemLanguage if cond => {
-                        let locale = locale_from_environment();
-                        cond = SystemLanguage::from_attribute(value, &locale)
-                            .map(|SystemLanguage(res)| res)?;
+                        cond = SystemLanguage::from_attribute(
+                            value,
+                            &(locale_from_environment().map_err(|e| ValueErrorKind::Value(e))?),
+                        )
+                        .map(|SystemLanguage(res)| res)?;
                     }
 
                     _ => {}
@@ -653,100 +657,26 @@ impl Node {
     }
 }
 
-/// Prevent stack overflow when recursively dropping nodes
-///
-/// Dropping nodes is recursive, since a node owns strong references
-/// to its next sibling and its first child.  When there is an SVG
-/// with a flat hierarchy of a few hundred thousand elements,
-/// recursively dropping these siblings can cause stack overflow.
-///
-/// Here, we convert recursion to an explicit heap-allocated stack of
-/// nodes that need to be dropped.  This technique is borrowed from
-/// [kuchiki]'s tree implementation.
-///
-/// [kuchiki]: https://github.com/kuchiki-rs/kuchiki/blob/master/src/tree.rs
-impl Drop for Node {
-    fn drop(&mut self) {
-        let mut stack = Vec::new();
-
-        if let Some(rc) = take_if_unique_strong(&self.first_child) {
-            non_recursive_drop_unique_rc(rc, &mut stack);
-        }
-
-        if let Some(rc) = take_if_unique_strong(&self.next_sib) {
-            non_recursive_drop_unique_rc(rc, &mut stack);
-        }
-
-        fn non_recursive_drop_unique_rc(mut rc: Rc<Node>, stack: &mut Vec<Rc<Node>>) {
-            loop {
-                if let Some(child) = take_if_unique_strong(&rc.first_child) {
-                    stack.push(rc);
-                    rc = child;
-                    continue;
-                }
-
-                if let Some(sibling) = take_if_unique_strong(&rc.next_sib) {
-                    rc = sibling;
-                    continue;
-                }
-
-                if let Some(parent) = stack.pop() {
-                    rc = parent;
-                    continue;
-                }
-
-                return;
-            }
-        }
-    }
-}
-
-/// Return `Some` if the `NodeRef` is the only strong reference count
-///
-/// Note that this leaves the tree in a partially inconsistent state, since
-/// the weak references to the node referenced by `r` will now point to
-/// an unlinked node.
-fn take_if_unique_strong(r: &RefCell<Option<Rc<Node>>>) -> Option<Rc<Node>> {
-    let mut r = r.borrow_mut();
-
-    let has_single_ref = match *r {
-        None => false,
-        Some(ref rc) if Rc::strong_count(rc) > 1 => false,
-        Some(_) => true,
-    };
-
-    if has_single_ref {
-        r.take()
-    } else {
-        None
-    }
-}
-
-pub fn node_ptr_to_weak(raw_parent: *const RsvgNode) -> Option<Weak<Node>> {
-    if raw_parent.is_null() {
-        None
-    } else {
-        let p: &RsvgNode = unsafe { &*raw_parent };
-        Some(Rc::downgrade(&p.clone()))
-    }
-}
-
-pub fn boxed_node_new(
+pub fn node_new(
     node_type: NodeType,
-    raw_parent: *const RsvgNode,
+    parent: Option<&RsvgNode>,
     element_name: &str,
     id: Option<&str>,
     class: Option<&str>,
     node_impl: Box<NodeTrait>,
-) -> *mut RsvgNode {
-    box_node(Rc::new(Node::new(
+) -> RsvgNode {
+    Rc::new(Node::new(
         node_type,
-        node_ptr_to_weak(raw_parent),
+        if let Some(parent) = parent {
+            Some(Rc::downgrade(parent))
+        } else {
+            None
+        },
         element_name,
         id,
         class,
         node_impl,
-    )))
+    ))
 }
 
 impl Children {
@@ -804,26 +734,6 @@ pub fn box_node(node: RsvgNode) -> *mut RsvgNode {
     Box::into_raw(Box::new(node))
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_get_parent(raw_node: *const RsvgNode) -> *const RsvgNode {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    match node.get_parent() {
-        None => ptr::null(),
-
-        Some(node) => box_node(node),
-    }
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_ref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    box_node(node.clone())
-}
-
 #[no_mangle]
 pub extern "C" fn rsvg_node_unref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
     if !raw_node.is_null() {
@@ -834,81 +744,10 @@ pub extern "C" fn rsvg_node_unref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
     ptr::null_mut()
 }
 
-#[no_mangle]
-pub extern "C" fn rsvg_node_add_child(raw_node: *mut RsvgNode, raw_child: *const RsvgNode) {
-    assert!(!raw_node.is_null());
-    assert!(!raw_child.is_null());
-    let node: &mut RsvgNode = unsafe { &mut *raw_node };
-    let child: &RsvgNode = unsafe { &*raw_child };
-
-    node.add_child(child);
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_find_last_chars_child(
-    raw_node: *const RsvgNode,
-    out_accept_chars: *mut glib_sys::gboolean,
-) -> *mut RsvgNode {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    let accept_chars = node.accept_chars();
-
-    assert!(!out_accept_chars.is_null());
-    unsafe {
-        *out_accept_chars = accept_chars.to_glib();
-    }
-
-    if accept_chars {
-        if let Some(chars) = node.find_last_chars_child() {
-            return box_node(chars);
-        }
-    }
-
-    ptr::null_mut()
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_children_iter_begin(raw_node: *const RsvgNode) -> *mut Children {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    Box::into_raw(Box::new(node.children()))
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_children_iter_end(iter: *mut Children) {
-    assert!(!iter.is_null());
-
-    unsafe { Box::from_raw(iter) };
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_children_iter_next(
-    iter: *mut Children,
-    out_child: *mut *mut RsvgNode,
-) -> glib_sys::gboolean {
-    assert!(!iter.is_null());
-
-    let iter = unsafe { &mut *iter };
-    if let Some(child) = iter.next() {
-        unsafe {
-            *out_child = box_node(child);
-        }
-        true.to_glib()
-    } else {
-        unsafe {
-            *out_child = ptr::null_mut();
-        }
-        false.to_glib()
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
     use handle::RsvgHandle;
-    use std::mem;
     use std::rc::Rc;
 
     struct TestNodeImpl {}
@@ -919,6 +758,13 @@ mod tests {
         }
     }
 
+    fn rsvg_node_ref(raw_node: *mut RsvgNode) -> *mut RsvgNode {
+        assert!(!raw_node.is_null());
+        let node: &RsvgNode = unsafe { &*raw_node };
+
+        box_node(node.clone())
+    }
+
     #[test]
     fn node_refs_and_unrefs() {
         let node = Rc::new(Node::new(
@@ -1055,45 +901,4 @@ mod tests {
         assert!(children.next().is_none());
         assert!(children.next_back().is_none());
     }
-
-    #[test]
-    fn node_children_iterator_c() {
-        let node = Rc::new(Node::new(
-            NodeType::Path,
-            None,
-            "path",
-            None,
-            None,
-            Box::new(TestNodeImpl {}),
-        ));
-
-        let child = Rc::new(Node::new(
-            NodeType::Path,
-            Some(Rc::downgrade(&node)),
-            "path",
-            None,
-            None,
-            Box::new(TestNodeImpl {}),
-        ));
-
-        let second_child = Rc::new(Node::new(
-            NodeType::Path,
-            Some(Rc::downgrade(&node)),
-            "path",
-            None,
-            None,
-            Box::new(TestNodeImpl {}),
-        ));
-
-        node.add_child(&child);
-        node.add_child(&second_child);
-
-        let iter = rsvg_node_children_iter_begin(&node);
-        let mut c = unsafe { mem::uninitialized() };
-
-        let result: bool = from_glib(rsvg_node_children_iter_next(iter, &mut c));
-        assert_eq!(result, true);
-        assert!(Rc::ptr_eq(unsafe { &*c }, &child));
-        rsvg_node_unref(c);
-    }
 }
