@@ -1,8 +1,5 @@
-use libc;
 use pango::{self, ContextExt, LayoutExt};
-use std;
 use std::cell::{Cell, RefCell};
-use std::str;
 
 use attributes::Attribute;
 use drawing_ctx::DrawingCtx;
@@ -10,7 +7,7 @@ use error::RenderingError;
 use font_props::FontWeightSpec;
 use handle::RsvgHandle;
 use length::*;
-use node::{boxed_node_new, CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
+use node::{CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
 use parsers::parse;
 use property_bag::PropertyBag;
 use space::{xml_space_normalize, NormalizeDefault, XmlSpaceNormalize};
@@ -55,14 +52,18 @@ pub struct NodeChars {
 }
 
 impl NodeChars {
-    fn new() -> NodeChars {
+    pub fn new() -> NodeChars {
         NodeChars {
             string: RefCell::new(String::new()),
             space_normalized: RefCell::new(None),
         }
     }
 
-    fn append(&self, s: &str) {
+    pub fn get_string(&self) -> String {
+        self.string.borrow().clone()
+    }
+
+    pub fn append(&self, s: &str) {
         self.string.borrow_mut().push_str(s);
         *self.space_normalized.borrow_mut() = None;
     }
@@ -106,7 +107,7 @@ impl NodeChars {
         let layout = self.create_layout(node, values, draw_ctx);
         let (width, _) = layout.get_size();
 
-        *length += f64::from(width) / f64::from(pango::SCALE);
+        *length = f64::from(width) / f64::from(pango::SCALE);
     }
 
     fn render(
@@ -581,7 +582,6 @@ fn create_pango_layout(
     ));
 
     let layout = pango::Layout::new(&pango_context);
-    layout.set_auto_dir(false);
     layout.set_font_description(&font_desc);
 
     let attr_list = pango::AttrList::new();
@@ -798,38 +798,3 @@ fn render_child(
 
     res
 }
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_chars_new(raw_parent: *const RsvgNode) -> *const RsvgNode {
-    boxed_node_new(
-        NodeType::Chars,
-        raw_parent,
-        "rsvg_chars",
-        None,
-        None,
-        Box::new(NodeChars::new()),
-    )
-}
-
-#[no_mangle]
-pub extern "C" fn rsvg_node_chars_append(
-    raw_node: *const RsvgNode,
-    text: *const libc::c_char,
-    len: isize,
-) {
-    assert!(!raw_node.is_null());
-    let node: &RsvgNode = unsafe { &*raw_node };
-
-    assert!(node.get_type() == NodeType::Chars);
-    assert!(!text.is_null());
-    assert!(len >= 0);
-
-    // libxml2 already validated the incoming string as UTF-8.  Note that
-    // it is *not* nul-terminated; this is why we create a byte slice first.
-    let bytes = unsafe { std::slice::from_raw_parts(text as *const u8, len as usize) };
-    let utf8 = unsafe { str::from_utf8_unchecked(bytes) };
-
-    node.with_impl(|chars: &NodeChars| {
-        chars.append(utf8);
-    });
-}
