@@ -7,6 +7,8 @@ use std::rc::Rc;
 use node::{box_node, Node, NodeType, RsvgNode};
 use state::ComputedValues;
 
+pub enum RsvgTree {}
+
 pub struct Tree {
     pub root: Rc<Node>,
     already_cascaded: Cell<bool>,
@@ -34,40 +36,36 @@ impl Tree {
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_tree_new(root: *const RsvgNode) -> *mut Tree {
-    assert!(!root.is_null());
-    let root: &RsvgNode = unsafe { &*root };
-
-    Box::into_raw(Box::new(Tree::new(root)))
-}
-
-#[no_mangle]
-pub unsafe extern "C" fn rsvg_tree_free(tree: *mut Tree) {
+pub extern "C" fn rsvg_tree_free(tree: *mut RsvgTree) {
     if !tree.is_null() {
-        Box::from_raw(tree);
+        let tree = unsafe { &mut *(tree as *mut Tree) };
+        let _ = unsafe { Box::from_raw(tree) };
     }
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_tree_cascade(tree: *const Tree) {
+pub extern "C" fn rsvg_tree_cascade(tree: *const RsvgTree) {
     assert!(!tree.is_null());
-    let tree = unsafe { &*tree };
+    let tree = unsafe { &*(tree as *const Tree) };
 
     tree.cascade();
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_tree_get_root(tree: *const Tree) -> *mut RsvgNode {
+pub extern "C" fn rsvg_tree_get_root(tree: *const RsvgTree) -> *mut RsvgNode {
     assert!(!tree.is_null());
-    let tree = unsafe { &*tree };
+    let tree = unsafe { &*(tree as *const Tree) };
 
     box_node(tree.root.clone())
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_tree_is_root(tree: *const Tree, node: *mut RsvgNode) -> glib_sys::gboolean {
+pub extern "C" fn rsvg_tree_is_root(
+    tree: *const RsvgTree,
+    node: *mut RsvgNode,
+) -> glib_sys::gboolean {
     assert!(!tree.is_null());
-    let tree = unsafe { &*tree };
+    let tree = unsafe { &*(tree as *const Tree) };
 
     assert!(!node.is_null());
     let node: &RsvgNode = unsafe { &*node };
@@ -76,9 +74,9 @@ pub extern "C" fn rsvg_tree_is_root(tree: *const Tree, node: *mut RsvgNode) -> g
 }
 
 #[no_mangle]
-pub extern "C" fn rsvg_tree_root_is_svg(tree: *const Tree) -> glib_sys::gboolean {
+pub extern "C" fn rsvg_tree_root_is_svg(tree: *const RsvgTree) -> glib_sys::gboolean {
     assert!(!tree.is_null());
-    let tree = unsafe { &*tree };
+    let tree = unsafe { &*(tree as *const Tree) };
 
     tree.root_is_svg().to_glib()
 }
